<?php

require_once __DIR__."/../vendor/autoload.php";

use NoccyLabs\React\Daemon\Daemon;
use NoccyLabs\React\Daemon\ModuleInterface;
use NoccyLabs\React\Daemon\Attributes\Help;
use NoccyLabs\React\Daemon\Attributes\OnEvent;
use NoccyLabs\React\Daemon\Attributes\Option;

#[Help(
    name:'some-daemon',
    summary:'This is a useful daemon.'
)]
class SomeDaemon extends Daemon
{
    #[Option(short:'h', long:'help', entrypoint:'help', help:"Show help")]
    public bool $showHelp = false;

    #[Option(short:'c', env:'APP_CONFIG', help:"Set config file")]
    public ?string $configFile = null;

    #[OnEvent('@start')]
    protected function onStart()
    {
        echo "onStart\n";
        $this->addNamedInterval("ping", 5, function () {
            $this->invokeEvent('app.ping');
        });
        $this->loadModule("web", new WebServer());
    }

    #[OnEvent('@stop')]
    protected function onStop()
    {
        echo "onStop\n";
    }

    #[OnEvent('app.ping')]
    protected function ping(): void
    {
        echo "PIng!\n";
    }
}

class WebServer implements ModuleInterface
{
    public function onLoad(): void
    {
        echo "web: start\n";
    }   

    public function onUnload(): void
    {
        echo "web: stop\n";
    }

    public function onEvent(string $event, mixed $data): void
    {
        printf("web: event %s (%s)\n", $event, json_encode($data));
    }
}

Daemon::boot(new SomeDaemon());
